"""
Client to submit jobs to and retrieve results from a remote ngscheck service.

The JSON manifest files used have the following simple form:
[
    {
        "jobid": <job_1>,
        "filepath": <input_filepath_1>
    },
    {
        "jobid": <job_2>,
        "filepath": <input_filepath_2>
    },
    ...
]
"""
import argparse
import getpass
import logging
import sys

from . import api


LOGGER = logging.getLogger(__name__)


def main():
    """
    Top-level control flow.
    """
    args = get_args(sys.argv[1:])
    logging.basicConfig(level=args.loglevel)
    exit_code = 0
    try:
        if args.password_file:
            password = args.password_file.read().strip()
            args.password_file.close()
        else:
            password = args.password or getpass.getpass("Password: ")
        token = api.get_access_token(args.email, password)
        userid = api.get_userid(token)
        command = args.subcommand
        if command == 'submit':
            api.submit(args.files, args.manifest_file, userid, token)
        elif command == 'get_status':
            api.get_status(args.manifest_file, userid, token)
        elif command == 'get_results':
            # get_status first to get latest status
            api.get_status(args.manifest_file, userid, token)
            api.get_results(args.manifest_file, userid, token)
        elif command == 'get_userid':
            print(userid)
        else:
            raise NotImplementedError(command)

    except Exception as ex:
        LOGGER.exception(ex)
        exit_code = 1
    LOGGER.info("exit_code: %s", exit_code)
    sys.exit(exit_code)


def get_args(args):
    """
    Parse arguments from the command line.

    Args:
        args: options & arguments from the command line, e.g. sys.argv[1:]

    Returns:
        argparse.Namespace
    """
    parser = argparse.ArgumentParser(description=__doc__)

    common_args = parser.add_argument_group('common arguments')
    common_args.add_argument(
        "--email",
        required=True,
        help="email used to sign up to the service")
    password_group = common_args.add_mutually_exclusive_group()
    password_group.add_argument(
        "--password",
        help="using --password-file or interactive use is safer")
    password_group.add_argument(
        "--password-file",
        type=argparse.FileType('r'),
        help="if neither --password nor --password-file is supplied a "
        "password will be prompted for interactively")
    common_args.add_argument(
        "--verbose",
        action="store_const",
        dest="loglevel",
        const=logging.INFO,
        default=logging.WARNING,
        help="output more logging (INFO level)")

    subparsers = parser.add_subparsers(
        dest="subcommand",
        required=True,
        help="SUBCOMMAND -h for subcommand help")
    subparsers.add_parser(
        "get_userid",
        help="return user identifier associated with this account (FYI only)")

    submit_p = subparsers.add_parser(
        "submit",
        help="submit files to be processed and generate a job manifest")
    submit_p.add_argument(
        "files",
        metavar="FILE",
        type=argparse.FileType('rb'),
        nargs='+')
    submit_p.add_argument(
        "--manifest-file",
        required=True,
        # exclusive write (fail if exists)
        type=argparse.FileType('x'))

    get_status_p = subparsers.add_parser(
        "get_status",
        help="check and update status of jobs listed in a manifest file")
    get_status_p.add_argument(
        "--manifest-file",
        required=True,
        type=argparse.FileType('r'))

    get_results_p = subparsers.add_parser(
        "get_results",
        help="get results of successful jobs listed in a manifest file")
    get_results_p.add_argument(
        "--manifest-file",
        required=True,
        type=argparse.FileType('r'))

    return parser.parse_args(args)


if __name__ == "__main__":
    main()



"""
Python method wrappers around ngscheck service REST API calls.

The JSON manifest files used have the following simple form:
[
    {
        "jobid": <job_1>,
        "filepath": <input_filepath_1>
    },
    {
        "jobid": <job_2>,
        "filepath": <input_filepath_2>
    },
    ...
]
"""
import json
import logging
import os.path
import uuid

import boto3
from botocore import UNSIGNED
from botocore.client import Config
import requests


APIBASE = "https://f4b45zm2fc.execute-api.ap-southeast-2.amazonaws.com/v1"
APIRESULTS = f"{APIBASE}/pickup"
APISTATUS = f"{APIBASE}/status"
APISUBMIT = f"{APIBASE}/submit"
APIUSERID = f"{APIBASE}/userid"
# ngscheck-service-client
CLIENTID = '7n8nbvr1slgml6nv5221pn1k65'
# use boto anonymously
COGNITO = boto3.client('cognito-idp', region_name='ap-southeast-2',
                       config=Config(signature_version=UNSIGNED))
NGSCHECK_EXTENSIONS = ['.ngcbas.json', '.ngcbas.pdf', '.ngcbas.xml']
LOGGER = logging.getLogger(__name__)


def get_access_token(identifier, password):
    """
    Returns AuthenticationResult.AccessToken from Cognito
    """
    response = COGNITO.initiate_auth(
        AuthFlow='USER_PASSWORD_AUTH',
        AuthParameters={'USERNAME': identifier, 'PASSWORD': password},
        ClientId=CLIENTID)
    LOGGER.info("auth response OK")
    return response['AuthenticationResult']['AccessToken']


def get_results(manifest_file, userid, token):
    """
    Get results for all SUCCEEDED jobs listed in the manifest.
    """
    try:
        manifest = json.loads(manifest_file.read())
        manifest_file.close()
    # if get_status was called first, manifest_file is closed
    except ValueError:
        with open(manifest_file.name) as reopened:
            manifest = json.loads(reopened.read())

    succeeded = [job for job in manifest if job['status'] == 'SUCCEEDED']
    failed = [job for job in manifest if job['status'].startswith('FAILED')]
    if len(succeeded) + len(failed) != len(manifest):
        LOGGER.warning("There are unfinished jobs. Aborting")
        return
    if failed:
        LOGGER.warning("There are failed jobs")
        LOGGER.warning(json.dumps(failed, indent=2))

    for job in succeeded:
        jobid = job['jobid']
        infilepath = job['filepath']
        key = os.path.basename(infilepath)
        for ext in NGSCHECK_EXTENSIONS:
            response = requests.get(
                f"{APIRESULTS}/{userid}/{jobid}/{key}{ext}",
                headers={'Authorization': token})
            response.raise_for_status()
            outfilepath = infilepath + ext
            with open(outfilepath, 'xb') as outfile:
                outfile.write(response.content)
            LOGGER.info(outfilepath)


def get_status(manifest_file, userid, token):
    """
    Get and update the status of the jobs listed in the manifest.
    """
    manifest = json.loads(manifest_file.read())
    manifest_file.close()
    for job in manifest:
        jobid = job['jobid']
        response = requests.get(
            f"{APISTATUS}/{userid}/{jobid}/STATUS",
            headers={'Authorization': token})
        response.raise_for_status()
        job.update({"status": response.text})
    manifest_str = json.dumps(manifest, indent=2)
    with open(manifest_file.name, 'w') as manifest_update:
        manifest_update.write(manifest_str)
    LOGGER.info(manifest_str)


def get_userid(token):
    """
    Returns userid from the token, in case it isn't known already.
    """
    response = requests.get(
        f"{APIUSERID}",
        headers={'Authorization': token})
    return response.text


def submit(files, manifest_file, userid, token):
    """
    Generate a job manifest and submit some files for processing.
    """
    manifest = [{'jobid': str(uuid.uuid4()), 'filepath': f.name}
                for f in files]
    manifest_str = json.dumps(manifest, indent=2)
    manifest_file.write(manifest_str)
    manifest_file.close()
    for infile, job in zip(files, manifest):
        jobid = job['jobid']
        key = os.path.basename(job['filepath'])
        response = requests.put(
            f"{APISUBMIT}/{userid}/{jobid}/{key}",
            data=infile.read(),
            headers={'Authorization': token})
        response.raise_for_status()
    LOGGER.info(manifest_str)



#!/usr/bin/env python3
"""
setuptools script

To make source and binary distribution for upload:
> python3 setup.py sdist bdist_wheel

"""
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    description="client for remote NGSCheck service",
    entry_points={
        'console_scripts': [
            'ngscheck_client = ngscheck_client.cli:main'
        ]
    },
    install_requires=[
        'boto3',
        'requests'
    ],
    long_description = long_description,
    long_description_content_type = "text/markdown",
    name="ngscheck_client",
    packages=find_packages(),
    python_requires=">=3.6",
    version="1.1.2"
)
